from apistar import types, validators


class PageNumberMeta(types.Type):
    page = validators.Integer(title="page", description="Current page number")
    page_size = validators.Integer(title="page_size", description="Page size")
    count = validators.Integer(title="count", description="Total number of items", allow_null=True)


class PageNumberSchema(types.Type):
    data = validators.Array()
    meta = PageNumberMeta


class LimitOffsetMeta(types.Type):
    limit = validators.Integer(title="limit", description="Number of retrieved items")
    offset = validators.Integer(title="offset", description="Collection offset")
    count = validators.Integer(title="count", description="Total number of items", allow_null=True)


class LimitOffsetSchema(types.Type):
    data = validators.Array()
    meta = LimitOffsetMeta



import typing

from apistar.http import JSONResponse
from apistar_pagination.schemas import LimitOffsetSchema, PageNumberSchema


class PageNumberResponse(JSONResponse):
    """
    Response paginated based on a page number and a page size.

    First 10 elements:
        /resource?page=1

    Third 10 elements:
        /resource?page=3

    First 20 elements:
        /resource?page=1&page_size=20
    """

    default_page_size = 10

    def __init__(
        self,
        page: typing.Optional[typing.Union[int, str]] = None,
        page_size: typing.Optional[typing.Union[int, str]] = None,
        count: typing.Optional[bool] = True,
        **kwargs
    ):
        self.page_number = int(page) if page is not None else 1
        self.page_size = int(page_size) if page_size is not None else self.default_page_size
        self.count = count
        print("Bar:", count)
        super().__init__(**kwargs)

    def render(self, content: typing.Sequence):
        init = (self.page_number - 1) * self.page_size
        end = self.page_number * self.page_size
        return super().render(
            PageNumberSchema(
                {
                    "meta": {
                        "page": self.page_number,
                        "page_size": self.page_size,
                        "count": len(content) if self.count else None,
                    },
                    "data": content[init:end],
                }
            )
        )


class LimitOffsetResponse(JSONResponse):
    """
    Response paginated based on a limit of elements and an offset.

    First 10 elements:
        /resource?offset=0&limit=10

    Elements 20-30:
        /resource?offset=20&limit=10
    """

    default_limit = 10

    def __init__(
        self,
        offset: typing.Optional[typing.Union[int, str]] = None,
        limit: typing.Optional[typing.Union[int, str]] = None,
        count: typing.Optional[bool] = True,
        **kwargs
    ):
        self.offset = int(offset) if offset is not None else 0
        self.limit = int(limit) if limit is not None else self.default_limit
        self.count = count
        super().__init__(**kwargs)

    def render(self, content: typing.Sequence):
        init = self.offset
        end = self.offset + self.limit
        return super().render(
            LimitOffsetSchema(
                {
                    "meta": {"limit": self.limit, "offset": self.offset, "count": len(content) if self.count else None},
                    "data": content[init:end],
                }
            )
        )



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['apistar_pagination']

package_data = \
{'': ['*']}

install_requires = \
['apistar>=0.5.30,<0.6.0']

setup_kwargs = {
    'name': 'apistar-pagination',
    'version': '0.4.0',
    'description': 'Pagination tools for API Star.',
    'long_description': '# API Star Pagination\n[![Build Status](https://travis-ci.org/PeRDy/apistar-pagination.svg?branch=master)](https://travis-ci.org/PeRDy/apistar-pagination)\n[![codecov](https://codecov.io/gh/PeRDy/apistar-pagination/branch/master/graph/badge.svg)](https://codecov.io/gh/PeRDy/apistar-pagination)\n[![PyPI version](https://badge.fury.io/py/apistar-pagination.svg)](https://badge.fury.io/py/apistar-pagination)\n\n* **Version:** 0.4.0\n* **Status:** Production/Stable\n* **Author:** José Antonio Perdiguero López\n\nPagination tools for API Star.\n\n## Features\n* Page number pagination.\n* Limit-offset pagination.\n\n## Quick start\nInstall API star Pagination:\n\n```bash\npip install apistar-pagination\n```\n\nUse paginated response in your views:\n\n### Page number pagination\n\n```python\nfrom apistar_pagination import PageNumberResponse\n\ndef page_number(page: http.QueryParam, page_size: http.QueryParam) -> typing.List[int]:\n    collection = range(10)  # Get your whole collection instead of a list of numbers\n\n    return PageNumberResponse(page=page, page_size=page_size, content=collection)\n```\n\n### Limit-offset pagination\n\n```python\nfrom apistar_pagination import LimitOffsetResponse\n\ndef limit_offset(offset: http.QueryParam, limit: http.QueryParam) -> typing.List[int]:\n    collection = range(10)  # Get your whole collection instead of a list of numbers\n\n    return LimitOffsetResponse(offset=offset, limit=limit, content=collection)\n```\n',
    'author': 'José Antonio Perdiguero López',
    'author_email': 'perdy@perdy.io',
    'url': 'https://github.com/PeRDy/apistar-pagination',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.5,<4.0',
}


setup(**setup_kwargs)
